@@ -42,7 +42,6 @@
 #include <gtksourceview/gtksourcelanguage.h>
 #include <gtksourceview/gtksourcelanguagesmanager.h>
 
-
 #include <gedit/gedit-prefs-manager.h>
 
 #include "gedit-preferences-dialog.h"
@@ -70,11 +69,11 @@ struct _GeditPreferencesDialog
 	/* Font & Colors */
 	GtkWidget	*default_font_checkbutton;
 	GtkWidget	*default_colors_checkbutton;
-	GtkWidget	*fontpicker;
-	GtkWidget	*text_colorpicker;
-	GtkWidget	*background_colorpicker;
-	GtkWidget	*sel_text_colorpicker;
-	GtkWidget	*selection_colorpicker;
+	GtkWidget	*font_button;
+	GtkWidget	*text_colorbutton;
+	GtkWidget	*background_colorbutton;
+	GtkWidget	*seltext_colorbutton;
+	GtkWidget	*selection_colorbutton;
 	GtkWidget	*colors_table;
 	GtkWidget	*font_hbox;
 	
@@ -121,9 +120,9 @@ struct _GeditPreferencesDialog
 	GtkWidget	*underline_togglebutton;
 	GtkWidget	*strikethrough_togglebutton;
 	GtkWidget	*foreground_checkbutton;
-	GtkWidget	*foreground_colorpicker;
+	GtkWidget	*foreground_colorbutton;
 	GtkWidget	*background_checkbutton;
-	GtkWidget	*background_colorpicker_2;
+	GtkWidget	*background_colorbutton_2;
 	GtkWidget	*reset_button;
 
 	/* Plugins manager */
@@ -131,52 +130,6 @@ struct _GeditPreferencesDialog
 };
 
 
-static void dialog_destroyed (GtkObject *obj,  void **dialog_pointer);
-
-/* TODO: monitor changes of the font size - Paolo */
-#define DEFAULT_FONT_SIZE 10
-
-static gint 
-get_desktop_default_font_size ()
-{
-	GConfClient *gconf_client = NULL;
-	gint res;
-	gchar *font;
-	
-	PangoFontDescription *desc;
-
-	gedit_debug (DEBUG_PREFS, "");
-
-	gconf_client = gconf_client_get_default ();
-	if (gconf_client == NULL)
-	{
-		return DEFAULT_FONT_SIZE;
-	}
-
-	font = gconf_client_get_string (gconf_client,
-				   	"/desktop/gnome/interface/font_name",
-				   	NULL);
-	if (font == NULL)
-		return DEFAULT_FONT_SIZE;
-	
-	gedit_debug (DEBUG_PREFS, "Font: %s", font);
-
-	desc = pango_font_description_from_string (font);
-
-	g_free (font);
-	g_return_val_if_fail (desc != NULL, DEFAULT_FONT_SIZE);
-
-	res = pango_font_description_get_size (desc) / PANGO_SCALE;
-
-	pango_font_description_free (desc);
-
-	g_object_unref (gconf_client);
-
-	gedit_debug (DEBUG_PREFS, "Size: %d", res);
-
-	return res;
-}
-
 static void
 dialog_destroyed (GtkObject  *obj,  
 		  void      **dialog_pointer)
@@ -486,8 +439,7 @@ right_margin_checkbutton_toggled (GtkToggleButton        *button,
 {
 	gboolean active;
 	
-	g_return_if_fail (button == 
-			GTK_TOGGLE_BUTTON (dlg->right_margin_checkbutton));
+	g_return_if_fail (button == GTK_TOGGLE_BUTTON (dlg->right_margin_checkbutton));
 
 	active = gtk_toggle_button_get_active (button);
 	
@@ -599,104 +551,105 @@ setup_view_page (GeditPreferencesDialog *dlg)
 
 }
 
+static void
+default_font_font_checkbutton_toggled (GtkToggleButton        *button,
+				       GeditPreferencesDialog *dlg)
+{
+	gedit_debug (DEBUG_PREFS, "");
+
+	g_return_if_fail (button == GTK_TOGGLE_BUTTON (dlg->default_font_checkbutton));
+
+	if (gtk_toggle_button_get_active (button))
+	{
+		gtk_widget_set_sensitive (dlg->font_hbox, FALSE);
+		gedit_prefs_manager_set_use_default_font (TRUE);
+	}
+	else
+	{
+		gtk_widget_set_sensitive (dlg->font_hbox, 
+					  gedit_prefs_manager_editor_font_can_set ());
+		gedit_prefs_manager_set_use_default_font (FALSE);
+	}
+}
+
 static void
 default_font_colors_checkbutton_toggled (GtkToggleButton        *button,
 					 GeditPreferencesDialog *dlg)
 {
 	gedit_debug (DEBUG_PREFS, "");
 
-	if (GTK_TOGGLE_BUTTON (dlg->default_font_checkbutton) == button)
+	g_return_if_fail (button == GTK_TOGGLE_BUTTON (dlg->default_colors_checkbutton));
+
+	if (gtk_toggle_button_get_active (button))
 	{
-		if (gtk_toggle_button_get_active (button))
-		{
-			gtk_widget_set_sensitive (dlg->font_hbox, FALSE);
-			gedit_prefs_manager_set_use_default_font (TRUE);
-		}
-		else
-		{
-			gtk_widget_set_sensitive (dlg->font_hbox, 
-						  gedit_prefs_manager_editor_font_can_set ());
-			gedit_prefs_manager_set_use_default_font (FALSE);
-		}
-
-		return;
+		gtk_widget_set_sensitive (dlg->colors_table, FALSE);
+		gedit_prefs_manager_set_use_default_colors (TRUE);
 	}
-
-	if (GTK_TOGGLE_BUTTON (dlg->default_colors_checkbutton) == button)
+	else
 	{
-		if (gtk_toggle_button_get_active (button))
-		{
-			gtk_widget_set_sensitive (dlg->colors_table, FALSE);
-			gedit_prefs_manager_set_use_default_colors (TRUE);
-		}
-		else
-		{
-			gedit_prefs_manager_set_use_default_colors (FALSE);
-			gtk_widget_set_sensitive (dlg->colors_table, 
-						  gedit_prefs_manager_background_color_can_set () &&
-					  	  gedit_prefs_manager_text_color_can_set () &&
-					  	  gedit_prefs_manager_selection_color_can_set () &&
-					  	  gedit_prefs_manager_selected_text_color_can_set ());
-		}
-
-		return;
+		gedit_prefs_manager_set_use_default_colors (FALSE);
+		gtk_widget_set_sensitive (dlg->colors_table, 
+					  gedit_prefs_manager_background_color_can_set () &&
+				  	  gedit_prefs_manager_text_color_can_set () &&
+				  	  gedit_prefs_manager_selection_color_can_set () &&
+				  	  gedit_prefs_manager_selected_text_color_can_set ());
 	}
-
-	g_return_if_fail (FALSE);
 }
 
 static void
-editor_font_picker_font_set (GnomeFontPicker        *gfp,
-			     const gchar            *font_name,
+editor_font_button_font_set (GtkFontButton          *font_button,
 			     GeditPreferencesDialog *dlg)
 {
+	const gchar *font_name;
+
 	gedit_debug (DEBUG_PREFS, "");
 
-	g_return_if_fail (gfp == GNOME_FONT_PICKER (dlg->fontpicker));
-	g_return_if_fail (font_name != NULL);
+	g_return_if_fail (font_button == GTK_FONT_BUTTON (dlg->font_button));
+
+	/* FIXME: Can this fail? Gtk docs are a bit terse... 21-02-2004 pbor */
+	font_name = gtk_font_button_get_font_name (font_button);
+	if (!font_name)
+	{
+		g_warning ("Could not get font name");
+		return;
+	}
 
 	gedit_prefs_manager_set_editor_font (font_name);
 }
 
 static void 
-editor_color_picker_color_set (GnomeColorPicker      *cp, 
-			       guint                   r, 
-			       guint                   g,
-			       guint                   b,
-			       guint                   a,
+editor_color_button_color_set (GtkColorButton         *button, 
 			       GeditPreferencesDialog *dlg)
 {
 	GdkColor color;
 
-	color.red = r;
-	color.green = g;
-	color.blue = b;
+	gtk_color_button_get_color (button, &color);
 
-	if (cp == GNOME_COLOR_PICKER (dlg->background_colorpicker))
-	{
-		gedit_prefs_manager_set_background_color (color);
-		return;
-	}
-
-	if (cp == GNOME_COLOR_PICKER (dlg->text_colorpicker))
+	if (button == GTK_COLOR_BUTTON (dlg->text_colorbutton))
 	{
 		gedit_prefs_manager_set_text_color (color);
 		return;
 	}
 
-	if (cp == GNOME_COLOR_PICKER (dlg->selection_colorpicker))
+	if (button == GTK_COLOR_BUTTON (dlg->background_colorbutton))
 	{
-		gedit_prefs_manager_set_selection_color (color);
+		gedit_prefs_manager_set_background_color (color);
 		return;
 	}
 
-	if (cp == GNOME_COLOR_PICKER (dlg->sel_text_colorpicker))
+	if (button == GTK_COLOR_BUTTON (dlg->seltext_colorbutton))
 	{
 		gedit_prefs_manager_set_selected_text_color (color);
 		return;
 	}
 
-	g_return_if_fail (FALSE);	
+	if (button == GTK_COLOR_BUTTON (dlg->selection_colorbutton))
+	{
+		gedit_prefs_manager_set_selection_color (color);
+		return;
+	}
+
+	g_return_if_reached ();	
 }
 
 static void
@@ -704,114 +657,75 @@ setup_font_colors_page (GeditPreferencesDialog *dlg)
 {
 	gboolean use_default_font;
 	gboolean use_default_colors;
-
-	GdkColor background_color;
-	GdkColor text_color;
-	GdkColor selection_color;
-	GdkColor selected_text_color;
-	
 	gchar *editor_font = NULL;
+	GdkColor color;
 
 	gedit_debug (DEBUG_PREFS, "");
-	
-	gtk_tooltips_set_tip (dlg->tooltips, dlg->fontpicker, 
-			_("Push this button to select the font to be used by the editor"), NULL);
 
-	gtk_tooltips_set_tip (dlg->tooltips, dlg->text_colorpicker, 
+	gtk_tooltips_set_tip (dlg->tooltips, dlg->font_button, 
+			_("Push this button to select the font to be used by the editor"), NULL);
+	gtk_tooltips_set_tip (dlg->tooltips, dlg->text_colorbutton, 
 			_("Push this button to configure text color"), NULL);
-	gtk_tooltips_set_tip (dlg->tooltips, dlg->background_colorpicker, 
+	gtk_tooltips_set_tip (dlg->tooltips, dlg->background_colorbutton, 
 			_("Push this button to configure background color"), NULL);
-	gtk_tooltips_set_tip (dlg->tooltips, dlg->sel_text_colorpicker, 
+	gtk_tooltips_set_tip (dlg->tooltips, dlg->seltext_colorbutton, 
 			_("Push this button to configure the color in which the selected "
 			  "text should appear"), NULL);
-	gtk_tooltips_set_tip (dlg->tooltips, dlg->selection_colorpicker, 
+	gtk_tooltips_set_tip (dlg->tooltips, dlg->selection_colorbutton, 
 			_("Push this button to configure the color in which the selected "
 			  "text should be marked"), NULL);
 
-	gedit_utils_set_atk_relation (dlg->fontpicker, dlg->default_font_checkbutton, 
+	gedit_utils_set_atk_relation (dlg->font_button, dlg->default_font_checkbutton, 
                                                           ATK_RELATION_CONTROLLED_BY);
-	gedit_utils_set_atk_relation (dlg->default_font_checkbutton, dlg->fontpicker, 
+	gedit_utils_set_atk_relation (dlg->default_font_checkbutton, dlg->font_button, 
                                                          ATK_RELATION_CONTROLLER_FOR);
-	
-	gnome_font_picker_fi_set_use_font_in_label (GNOME_FONT_PICKER (dlg->fontpicker),
-                                                    TRUE, 
-						    get_desktop_default_font_size ());
 
-	/* read config value */
+	/* read current config and setup initial state */
 	use_default_font = gedit_prefs_manager_get_use_default_font ();
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dlg->default_font_checkbutton),
+				      use_default_font);
+
 	use_default_colors = gedit_prefs_manager_get_use_default_colors ();
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dlg->default_colors_checkbutton),
+				      use_default_colors);
 
-	background_color = gedit_prefs_manager_get_background_color ();
-	text_color = gedit_prefs_manager_get_text_color ();
-	selection_color = gedit_prefs_manager_get_selection_color ();
-	selected_text_color = gedit_prefs_manager_get_selected_text_color ();
-	
 	editor_font = gedit_prefs_manager_get_editor_font ();
-
-	/* setup the initial states */ 
-	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (dlg->background_colorpicker),
-				    background_color.red,
-				    background_color.green,  
-				    background_color.blue, 0);
-
-	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (dlg->text_colorpicker),
-				    text_color.red,
-				    text_color.green,
-				    text_color.blue, 0);
-
-	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (dlg->selection_colorpicker),
-				    selection_color.red,
-				    selection_color.green,
-				    selection_color.blue, 0);
-
-	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (dlg->sel_text_colorpicker),
-				    selected_text_color.red,
-				    selected_text_color.green,
-				    selected_text_color.blue, 0);
-
 	if (editor_font != NULL)
-	{
-		gnome_font_picker_set_font_name (GNOME_FONT_PICKER (dlg->fontpicker),
-						 editor_font);
+		gtk_font_button_set_font_name (GTK_FONT_BUTTON (dlg->font_button),
+					       editor_font);
+	g_free (editor_font);
 
-		g_free (editor_font);
-	}
+	color = gedit_prefs_manager_get_text_color ();
+	gtk_color_button_set_color (GTK_COLOR_BUTTON (dlg->text_colorbutton),
+				    &color);
 
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON 
-			(dlg->default_font_checkbutton), use_default_font);
+	color = gedit_prefs_manager_get_background_color ();
+	gtk_color_button_set_color (GTK_COLOR_BUTTON (dlg->background_colorbutton),
+				    &color);
 
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON 
-			(dlg->default_colors_checkbutton), use_default_colors);
+	color = gedit_prefs_manager_get_selected_text_color ();
+	gtk_color_button_set_color (GTK_COLOR_BUTTON (dlg->seltext_colorbutton),
+				    &color);
+
+	color = gedit_prefs_manager_get_selection_color ();
+	gtk_color_button_set_color (GTK_COLOR_BUTTON (dlg->selection_colorbutton),
+				    &color);
 
 	/* Connect signals */
-	
 	g_signal_connect (G_OBJECT (dlg->default_font_checkbutton), "toggled", 
-			G_CALLBACK (default_font_colors_checkbutton_toggled), 
-			dlg);
-
+			  G_CALLBACK (default_font_font_checkbutton_toggled), dlg);
 	g_signal_connect (G_OBJECT (dlg->default_colors_checkbutton), "toggled", 
-			G_CALLBACK (default_font_colors_checkbutton_toggled), 
-			dlg);
-
-	g_signal_connect (G_OBJECT (dlg->fontpicker), "font_set", 
-			G_CALLBACK (editor_font_picker_font_set), 
-			dlg);
-
-	g_signal_connect (G_OBJECT (dlg->background_colorpicker), "color_set",
-			G_CALLBACK (editor_color_picker_color_set),
-			dlg);
-
-	g_signal_connect (G_OBJECT (dlg->text_colorpicker), "color_set",
-			G_CALLBACK (editor_color_picker_color_set),
-			dlg);
-
-	g_signal_connect (G_OBJECT (dlg->selection_colorpicker), "color_set",
-			G_CALLBACK (editor_color_picker_color_set),
-			dlg);
-
-	g_signal_connect (G_OBJECT (dlg->sel_text_colorpicker), "color_set",
-			G_CALLBACK (editor_color_picker_color_set),
-			dlg);
+			  G_CALLBACK (default_font_colors_checkbutton_toggled), dlg);
+	g_signal_connect (G_OBJECT (dlg->font_button), "font_set", 
+			  G_CALLBACK (editor_font_button_font_set), dlg);
+	g_signal_connect (G_OBJECT (dlg->text_colorbutton), "color_set",
+			  G_CALLBACK (editor_color_button_color_set), dlg);
+	g_signal_connect (G_OBJECT (dlg->background_colorbutton), "color_set",
+			  G_CALLBACK (editor_color_button_color_set), dlg);
+	g_signal_connect (G_OBJECT (dlg->seltext_colorbutton), "color_set",
+			  G_CALLBACK (editor_color_button_color_set), dlg);
+	g_signal_connect (G_OBJECT (dlg->selection_colorbutton), "color_set",
+			  G_CALLBACK (editor_color_button_color_set), dlg);
 
 	/* Set initial widget sensitivity */
 	gtk_widget_set_sensitive (dlg->default_font_checkbutton, 
@@ -952,25 +866,41 @@ styles_cb (GtkWidget              *treeview,
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dlg->foreground_checkbutton),
 				      style->mask & GTK_SOURCE_TAG_STYLE_USE_FOREGROUND);
 
-	/* FIXME: Set a sane default. Paolo */
-	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (dlg->foreground_colorpicker),
-				    style->foreground.red,
-				    style->foreground.green,
-				    style->foreground.blue,
-				    0);
-	gtk_widget_set_sensitive (dlg->foreground_colorpicker, 
+	if ((style->mask & GTK_SOURCE_TAG_STYLE_USE_FOREGROUND) == GTK_SOURCE_TAG_STYLE_USE_FOREGROUND)
+	{
+		gtk_color_button_set_color (GTK_COLOR_BUTTON (dlg->foreground_colorbutton),
+					    &style->foreground);
+	}
+	else
+	{
+		GdkColor text_color;
+		
+		text_color = gedit_prefs_manager_get_text_color ();
+		gtk_color_button_set_color (GTK_COLOR_BUTTON (dlg->foreground_colorbutton),
+					    &text_color);
+	}
+	
+	gtk_widget_set_sensitive (dlg->foreground_colorbutton, 
 				  style->mask & GTK_SOURCE_TAG_STYLE_USE_FOREGROUND);
 
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dlg->background_checkbutton),
 				      style->mask & GTK_SOURCE_TAG_STYLE_USE_BACKGROUND);
 	
-	/* FIXME: Set a sane default. Paolo */
-	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (dlg->background_colorpicker_2),
-				    style->background.red,
-				    style->background.green,
-				    style->background.blue,
-				    0);
-	gtk_widget_set_sensitive (dlg->background_colorpicker_2, 
+	if ((style->mask & GTK_SOURCE_TAG_STYLE_USE_BACKGROUND) == GTK_SOURCE_TAG_STYLE_USE_BACKGROUND)
+	{
+		gtk_color_button_set_color (GTK_COLOR_BUTTON (dlg->background_colorbutton_2),
+					    &style->background);
+	}
+	else
+	{
+		GdkColor background_color;
+		
+		background_color = gedit_prefs_manager_get_background_color ();
+		gtk_color_button_set_color (GTK_COLOR_BUTTON (dlg->background_colorbutton_2),
+					    &background_color);
+	}
+
+	gtk_widget_set_sensitive (dlg->background_colorbutton_2, 
 				  style->mask & GTK_SOURCE_TAG_STYLE_USE_BACKGROUND);
 
 	gtk_widget_set_sensitive (dlg->reset_button, !style->is_default);
@@ -1011,17 +941,59 @@ style_button_toggled (GtkToggleButton        *button,
 	new_style->strikethrough = gtk_toggle_button_get_active (
 					GTK_TOGGLE_BUTTON (dlg->strikethrough_togglebutton));
 
+	if (gtk_toggle_button_get_active (
+		GTK_TOGGLE_BUTTON (dlg->foreground_checkbutton)))
+	{
+		new_style->mask |= GTK_SOURCE_TAG_STYLE_USE_FOREGROUND;
+		gtk_color_button_get_color (GTK_COLOR_BUTTON (dlg->foreground_colorbutton),
+					    &new_style->foreground);
+		gtk_widget_set_sensitive (dlg->foreground_colorbutton,
+					  TRUE);
+	}
+	else
+	{
+		new_style->mask &= ~GTK_SOURCE_TAG_STYLE_USE_FOREGROUND;
+		gtk_widget_set_sensitive (dlg->foreground_colorbutton,
+					  FALSE);
+	}
+
+	if (gtk_toggle_button_get_active (
+		GTK_TOGGLE_BUTTON (dlg->background_checkbutton)))
+	{
+		new_style->mask |= GTK_SOURCE_TAG_STYLE_USE_BACKGROUND;
+		gtk_color_button_get_color (GTK_COLOR_BUTTON (dlg->background_colorbutton_2),
+					    &new_style->background);
+		gtk_widget_set_sensitive (dlg->background_colorbutton_2,
+					  TRUE);
+	}
+	else
+	{
+		new_style->mask &= ~GTK_SOURCE_TAG_STYLE_USE_BACKGROUND;
+		gtk_widget_set_sensitive (dlg->background_colorbutton_2,
+					  FALSE);
+	}
+
 	if (memcmp (style, new_style, sizeof (GtkSourceTagStyle)) != 0)
 	{
 		GtkSourceTagStyle *def_style;
 
-		gedit_language_set_tag_style (lang, id, new_style);
-
 		def_style = gtk_source_language_get_tag_default_style (lang, id);
 
+		if (!(new_style->mask & GTK_SOURCE_TAG_STYLE_USE_BACKGROUND))
+		{
+			new_style->background = def_style->background;
+		}
+
+		if (!(new_style->mask & GTK_SOURCE_TAG_STYLE_USE_FOREGROUND))
+		{
+			new_style->foreground = def_style->foreground;
+		}
+		
 		gtk_widget_set_sensitive (dlg->reset_button, 
 					  memcmp (new_style, def_style, sizeof (GtkSourceTagStyle)) != 0);
 
+		gedit_language_set_tag_style (lang, id, new_style);
+
 		gtk_source_tag_style_free (def_style);
 
 	}
@@ -1032,6 +1004,13 @@ style_button_toggled (GtkToggleButton        *button,
 	g_free (id);
 }
 
+static void
+style_color_set (GtkColorButton         *button,
+		 GeditPreferencesDialog *dlg)
+{
+	style_button_toggled (NULL, dlg);
+}
+
 static void
 reset_button_clicked (GtkButton              *button, 
 		      GeditPreferencesDialog *dlg)
@@ -1078,7 +1057,6 @@ setup_syntax_highlighting_page (GeditPreferencesDialog *dlg)
 	column = gtk_tree_view_column_new_with_attributes (_("Elements"), renderer,
 							   "text", NAME_COLUMN, NULL);
 
-	gtk_tree_view_column_set_sort_column_id (column, 0);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (dlg->tags_treeview), column);
 
 	/* Connect signals */
@@ -1125,6 +1103,14 @@ setup_syntax_highlighting_page (GeditPreferencesDialog *dlg)
 			  G_CALLBACK (style_button_toggled), dlg);
 	g_signal_connect (G_OBJECT (dlg->strikethrough_togglebutton), "toggled",
 			  G_CALLBACK (style_button_toggled), dlg);
+	g_signal_connect (G_OBJECT (dlg->foreground_checkbutton), "toggled",
+			  G_CALLBACK (style_button_toggled), dlg);
+	g_signal_connect (G_OBJECT (dlg->background_checkbutton), "toggled",
+			  G_CALLBACK (style_button_toggled), dlg);
+	g_signal_connect (G_OBJECT (dlg->foreground_colorbutton), "color_set",
+			  G_CALLBACK (style_color_set), dlg);
+	g_signal_connect (G_OBJECT (dlg->background_colorbutton_2), "color_set",
+			  G_CALLBACK (style_color_set), dlg);
 	g_signal_connect (G_OBJECT (dlg->reset_button), "clicked",
 			  G_CALLBACK (reset_button_clicked), dlg);
 }
@@ -1153,22 +1139,20 @@ get_preferences_dialog (GtkWindow *parent)
 
 	if (dialog != NULL)
 	{
-
-		gtk_window_present (GTK_WINDOW (dialog->dialog));
-
 		gtk_window_set_transient_for (GTK_WINDOW (dialog->dialog),
 					      parent);
+		gtk_window_present (GTK_WINDOW (dialog->dialog));
 
 		return dialog;
 	}
 
 	gui = glade_xml_new (GEDIT_GLADEDIR "gedit-preferences.glade2",
 			     "preferences_dialog", NULL);
-
 	if (!gui)
 	{
-		gedit_warning (_("Could not find '" GEDIT_GLADEDIR "gedit-preferences.glade2'.\n\n"
-			       "Reinstall gedit."), parent);
+		gedit_warning (parent,
+			       MISSING_FILE,
+			       GEDIT_GLADEDIR "gedit-preferences.glade2");
 		return NULL;
 	}
 
@@ -1181,12 +1165,12 @@ get_preferences_dialog (GtkWindow *parent)
 
 	dialog->default_font_checkbutton = glade_xml_get_widget (gui, "default_font_checkbutton");
 	dialog->default_colors_checkbutton = glade_xml_get_widget (gui, "default_colors_checkbutton");
-	
-	dialog->fontpicker = glade_xml_get_widget (gui, "fontpicker");
-	dialog->text_colorpicker = glade_xml_get_widget (gui, "text_colorpicker");
-	dialog->background_colorpicker = glade_xml_get_widget (gui, "background_colorpicker");
-	dialog->sel_text_colorpicker = glade_xml_get_widget (gui, "sel_text_colorpicker");
-	dialog->selection_colorpicker = glade_xml_get_widget (gui, "selection_colorpicker");
+
+	dialog->font_button = glade_xml_get_widget (gui, "font_button");
+	dialog->text_colorbutton = glade_xml_get_widget (gui, "text_colorbutton");
+	dialog->background_colorbutton = glade_xml_get_widget (gui, "background_colorbutton");
+	dialog->seltext_colorbutton = glade_xml_get_widget (gui, "seltext_colorbutton");
+	dialog->selection_colorbutton = glade_xml_get_widget (gui, "selection_colorbutton");
 
 	dialog->colors_table = glade_xml_get_widget (gui, "colors_table");
 	dialog->font_hbox = glade_xml_get_widget (gui, "font_hbox");	
@@ -1223,9 +1207,9 @@ get_preferences_dialog (GtkWindow *parent)
 	dialog->underline_togglebutton = glade_xml_get_widget (gui, "underline_togglebutton");
 	dialog->strikethrough_togglebutton = glade_xml_get_widget (gui, "strikethrough_togglebutton");
 	dialog->foreground_checkbutton = glade_xml_get_widget (gui, "foreground_checkbutton");
-	dialog->foreground_colorpicker = glade_xml_get_widget (gui, "foreground_colorpicker");
+	dialog->foreground_colorbutton = glade_xml_get_widget (gui, "foreground_colorbutton");
 	dialog->background_checkbutton = glade_xml_get_widget (gui, "background_checkbutton");
-	dialog->background_colorpicker_2 = glade_xml_get_widget (gui, "background_colorpicker_2");
+	dialog->background_colorbutton_2 = glade_xml_get_widget (gui, "background_colorbutton_2");
 	dialog->reset_button = glade_xml_get_widget (gui, "reset_button");
 
 	if (!dialog->dialog ||
@@ -1234,11 +1218,11 @@ get_preferences_dialog (GtkWindow *parent)
 	    !dialog->split_checkbutton ||
 	    !dialog->default_font_checkbutton ||
 	    !dialog->default_colors_checkbutton ||
-	    !dialog->fontpicker ||
-	    !dialog->text_colorpicker ||
-	    !dialog->background_colorpicker ||
-	    !dialog->sel_text_colorpicker ||
-	    !dialog->selection_colorpicker ||
+	    !dialog->font_button ||
+	    !dialog->text_colorbutton ||
+	    !dialog->background_colorbutton ||
+	    !dialog->seltext_colorbutton ||
+	    !dialog->selection_colorbutton ||
 	    !dialog->colors_table ||
 	    !dialog->font_hbox ||
 	    !dialog->right_margin_checkbutton ||
@@ -1266,14 +1250,14 @@ get_preferences_dialog (GtkWindow *parent)
 	    !dialog->underline_togglebutton ||
 	    !dialog->strikethrough_togglebutton ||
 	    !dialog->foreground_checkbutton ||
-	    !dialog->foreground_colorpicker ||
+	    !dialog->foreground_colorbutton ||
 	    !dialog->background_checkbutton ||
-	    !dialog->background_colorpicker_2 ||
+	    !dialog->background_colorbutton_2 ||
 	    !dialog->reset_button)
 	{
-		gedit_warning (_("Could not find the required widgets inside '" 
-			       GEDIT_GLADEDIR "gedit-preferences.glade2'.\n\n"
-			       "Reinstall gedit."), parent);
+		gedit_warning (parent,
+			       MISSING_WIDGETS,
+			       GEDIT_GLADEDIR "gedit-preferences.glade2");
 
 		if (!dialog->dialog)
 			gtk_widget_destroy (dialog->dialog);
@@ -1320,8 +1304,7 @@ gedit_show_preferences_dialog (GtkWindow *parent)
 	g_return_if_fail (parent != NULL);
 
 	dialog = get_preferences_dialog (parent);
-	
-	if (dialog == NULL) 
+	if (!dialog) 
 		return;
 
 	if (!GTK_WIDGET_VISIBLE (dialog->dialog))
